/// Validation object which constructs the properties needed for each element the jquery validation plugin
function ValidateClient( $ ) {
	this.config = {};
	this.$fb = $;
}

// Helper for cloning object
ValidateClient.prototype.clone = function(obj){
	
    if(obj == null || typeof(obj) != 'object')
        return obj;

    var temp = {};
    for(var key in obj)
        temp[key] = this.clone(obj[key]);

    return temp;
}

// Initializes the validation for the editor
ValidateClient.prototype.init_editor = function()
{	
	// Disables the submit of the form
	this.$fb("#fb-submit-button").click( function(){return false;}  );
};


// Initializes the validation for the client
ValidateClient.prototype.init_client = function( isPreview ) {
	
	var thisRef = this;
	
	// Removes the placeholder fallback in case existings
	this.$fb("#docContainer input:submit, #docContainer button:submit").click(function() {
		thisRef.$fb("#docContainer .placeholder").val("");
		
		// If we are working on sdrive (iframe or auto embedded), add the stats code see (empire:#1740)
		if( thisRef.$fb("#docContainer").attr('data-form') == 'automated' || thisRef.$fb("#docContainer").attr('data-form') == 'publish' ){

			// send the statistics when there are errors in the form
			if( !thisRef.$fb( '#docContainer' ).valid() ) {
				var action = thisRef.$fb('#docContainer').attr( 'action' );
				var numErrors = thisRef.$fb('#docContainer').validate().numberOfInvalids()
				action = action.replace( /(?:\.php)|\/$/i , '/fbapp/api/formchangestats.php?error=' + numErrors );
				thisRef.$fb.get( action );
			}
		}
		
		return true;
	});

	if( isPreview ) {
		this.$fb.validator.setDefaults({
			submitHandler: function( form ) {
				if( thisRef.$fb(form).valid() ) {
					alert("Submitted!");
				}
			} 
		});
	}
	
	
	this.$fb("#docContainer").validate({
		errorPlacement: function(error, element) {
		            offset = element.offset();
		            //error.insertBefore( element.parent().childNodes[0] )
					if( thisRef.$fb(element).attr( "type" ) == "checkbox" )
					{
						 thisRef.$fb(element).parent().parent().append( thisRef.$fb(error) );
					}
					else if( thisRef.$fb(element).attr( "id" ) == "recaptcha_response_field" )
					{
						thisRef.$fb(error).insertAfter( thisRef.$fb("#recaptcha_area") );
					}
					else
					{
						thisRef.$fb(error).insertAfter( thisRef.$fb(element) );
					}					
					
		            error.addClass('message');  // add a class to the wrapper
		        }
	});
	
	this.set_config( JSON.parse( data_validation ) ); 
	this.add_rules();  

}

// Sets the rules for the validation client.
ValidateClient.prototype.set_config = function( config )
{
	this.config = this.clone( config );	
};

// Sets the json configuration for the validation
ValidateClient.prototype.set_config_json = function( json_config )
{
	// if its undefined or it is not a string do nothing
	if( typeof json_config == "undefined" || json_config === null || typeof json_config != "string" ) { return false; }
	
	// parses the json and set it
	this.config = JSON.parse( json_config );
	return true;
};

// Adds the rule to the element
ValidateClient.prototype.add_rule_type = function( id, type )
{
	// If the argument is not an id we do nothing
	if( arguments.length != 2 ) { return false; }

	// In case there is no config for that element do nothing
	if( typeof this.config[id] == 'undefined' || this.config[id] === null ) { return false; }
	
	// In case there is no config or is empty for that element do nothing
	if( typeof this.config[id][type] == 'undefined' || this.config[id][type] === null || this.config[id][type] == "" ) { return false; }
	
	// Checks if the element really exists in the html and it is not the captcha
	if(  id != 'reCaptcha' && ! this.$fb(  '#' + id ).length ) { return false; }
	
	var thisRef = this;
	if( type == 'date_config' )
	{
		// We init the object as a datepicker, triggering the change each time a new date is selected
		this.$fb('#' + id ).attr('id', '').removeClass('hasDatepicker').removeData('datepicker').unbind().attr('id',id).datepicker({
			onClose: function(){
				thisRef.$fb("#docContainer").validate().element(this);
			},
			showAnim:''
		});
		
		// We loop through all the date rule definitions
		for( var date_type in this.config[id][type] )
		{
			// We check the type is really an element of the object
			if ( this.config[id][type].hasOwnProperty( date_type ) ) 
			{
				var daterule_obj = {};
				
				// adds the date picker rule for that element
				if( date_type == "minDate" || date_type == "maxDate" )
				{
					if( this.config[id][type][date_type] != "" )
						daterule_obj[ date_type ] = new Date( this.config[id][type][date_type] );
				}
				else
				{
					daterule_obj[ date_type ] = this.config[id][type][date_type];
				}
				this.$fb( '#' + id ).datepicker("option", date_type, daterule_obj[ date_type ] );
			}
		}		
	}
	// We set the captcha as a required value
	else if( type == 'captcha' )
	{
		if( this.config[id][type] && this.$fb( '#recaptcha_response_field' ).length )
		{
			this.$fb( '#recaptcha_response_field' ).rules( "add", { required: true } );
		}
	}
	// We don't treat label items as they are not part of validation for the client
	else
	{
		var rule_obj = {};
		rule_obj[ type ] = this.config[id][type];

		// If it is a checkbox and no message is establish we add the default error message for the minlenght
		if( this.$fb('#' + id).is("input[type='checkbox']") && type == 'minlength' )
		{
			var msg_obj_chk = {};
			msg_obj_chk[type] = "Please select at least {0} options.";
			rule_obj.messages = msg_obj_chk;
		}


		// We add the messages in case existing
		if( typeof( this.config[id].messages ) == 'string' )
		{
			var msg_obj = {};
			msg_obj[type] = this.config[id].messages;
			rule_obj.messages = msg_obj;
		}
		
	
		// adds the rule for that element
		this.$fb(  '#' + id ).rules( "add", rule_obj );
	}
	
	return true;	
};


// Adds the rules for a specific element 
ValidateClient.prototype.add_rule_id = function( id ) 
{
	// If the argument is not an id we do nothing
	if( arguments.length != 1 ) { return false; }
	
	// In case there is no config for that element do nothing
	if( typeof this.config[id] == 'undefined' || this.config[id] === null ) { return false; }

	// We loop through all the rule definitions
	for( var type in this.config[id] )
	{
		// We check the type is really an element of the object
		if ( this.config[id].hasOwnProperty(type) ) 
		{
			// We treat the rules per ID
			this.add_rule_type( id, type  );
		}
	}
	return true;
};


// This method add all the rules existing in the validation config.
ValidateClient.prototype.add_rules = function()
{
	// We loop through all the rule definitions
	for( var id in this.config )
	{
		if( this.config.hasOwnProperty(id) )
		{
			this.add_rule_id( id );
		}
	}
	
	return true;
};
